#%% Base imports
# NB: requires the installation of "tabulate" to run the entire script. Not necessary to obtain results though.
# --> To install, open Anaconda Prompt(if using conda) with adm. rights and type: "conda install tabulate" and press enter + Y

# Base imports
import numpy as np
import pandas as pd

import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import matplotlib.dates as mdates
import seaborn as sns

import statsmodels.formula.api as smf
import statsmodels.api as sm
from IPython.core.display import display, HTML
from tabulate import tabulate

# For plotting
import plotly.offline as offline
from plotly.offline import download_plotlyjs, init_notebook_mode, plot, iplot
import plotly.express as px
import plotly.graph_objs as go
offline.init_notebook_mode(connected=True)
pd.set_option('display.max_rows', 10)

# personal settings for optimized experience
np.set_printoptions(precision = 4, suppress=True, edgeitems=20, linewidth=100)
pd.set_option('display.float_format', lambda x: '%.4f' % x)
pd.set_option('display.max_rows', 500)
pd.set_option('display.min_rows', 500)
pd.set_option('display.max_columns', 20)
pd.options.mode.chained_assignment = None

from IPython.core.display import display, HTML
display(HTML("<style>.container { width:100% !important; }</style>"))

# https://www.youtube.com/watch?v=wIfjERirYZk
#import yfinance as yf

#%% Load the data

fnam_equity  = "Futures_E.csv"
fnam_bond    = "Bonds.csv"
fnam_comm    = "Commodities4.csv"
fnam_fx      = "FX.csv"

fnam_proxyE  = "Proxy_E.csv"
fnam_proxyFX = "Proxy_C.csv"
fnam_libor   = "Libor.csv"
fnam_carhart = "Carhart_new.csv"

equity_raw   = pd.read_csv(fnam_equity, delimiter=';', decimal=',')
bond_raw     = pd.read_csv(fnam_bond,   delimiter=';', decimal=',')
comm_raw     = pd.read_csv(fnam_comm,   delimiter=';', decimal=',')
fx_raw       = pd.read_csv(fnam_fx,     delimiter=';', decimal=',')

proxyE_raw   = pd.read_csv(fnam_proxyE,  delimiter=';', decimal=',')
proxyFX_raw  = pd.read_csv(fnam_proxyFX, delimiter=';', decimal=',')

libor        = pd.read_csv(fnam_libor, delimiter=';', decimal=',')
carhart_ret  = pd.read_csv(fnam_carhart, delimiter=';', decimal ='.' )

#Renaming the columns
eq_cols    = ['Date','FTSE','S&P 500', 'IBEX 35', 'AEX', 'TOPIX', 'MIB', 'DAX', 'CAC 40', 'SPI 200']
bond_cols  = ['Date','3-year AUS','10-year AUS','2-year EURO','5-year EURO','10-year EURO','30-year EURO', '10-year JP','10-year CAN','10-year UK','2-year US', '5-year US','10-year US', '30-year US']
comm_cols  = ['Date','Platinum', 'Silver', 'Gold','Heating Oil','Natural Gas','Unleaded','WTI Crude','Wheat','Soy Oil','Soy Meal','Soybean','Corn','Cattle','Lean Hogs','Gas Oil','Brent Crude','Coffee','Sugar','Cotton','Cocoa','Zinc','Nickel','Copper','Aluminium']
fx_cols    = ['Date', 'AUD-USD','EUR-USD','CAD-USD','JPY-USD','NOK-USD','NZD-USD','CHF-USD','GBP-USD','SEK-USD']

proxyE_cols   = ['Date', 'IBEX 35', 'TOPIX', 'MIB', 'DAX','CAC 40', 'SPI 200']
proxyFX_cols  = ['Date', 'AUD-USD','EUR-USD','CAD-USD','JPY-USD','NOK-USD','NZD-USD','CHF-USD','GBP-USD','SEK-USD']

libor_cols      = ['Date', 'Libor']
carhart_cols    = ['Date','SMB','HML', 'UMD']

equity_raw.columns = eq_cols
bond_raw.columns   = bond_cols
comm_raw.columns   = comm_cols
fx_raw.columns     = fx_cols

proxyE_raw.columns  = proxyE_cols
proxyFX_raw.columns = proxyFX_cols
libor.columns       = libor_cols
carhart_ret.columns     = carhart_cols

# Set the Date column as index
equity_raw.set_index('Date',inplace=True)
bond_raw.set_index('Date',inplace=True)
comm_raw.set_index('Date',inplace=True)
fx_raw.set_index('Date',inplace=True)

proxyE_raw.set_index('Date',inplace=True)
proxyFX_raw.set_index('Date',inplace=True)

libor.set_index('Date',inplace=True)
carhart_ret.set_index('Date',inplace=True)

#Convert carhart to datetime
carhart_ret.index = pd.to_datetime(carhart_ret.index)

# Delete unnecessairy variables
del fnam_bond
del fnam_carhart
del fnam_comm
del fnam_equity
del fnam_fx
del fnam_libor
del fnam_proxyE
del fnam_proxyFX

#%% 0.0 Cut the length of the data to 2009
equity_raw.drop(equity_raw.tail(3131).index,0,inplace=True)
bond_raw.drop(bond_raw.tail(3131).index,0,inplace=True)
comm_raw.drop(comm_raw.tail(3131).index,0,inplace=True)
fx_raw.drop(fx_raw.tail(3131).index,0,inplace=True)

proxyE_raw.drop(proxyE_raw.tail(3131).index,0,inplace=True)
proxyFX_raw.drop(proxyFX_raw.tail(3131).index,0,inplace=True)


carhart_ret.drop(carhart_ret.tail(113).index,0,inplace=True)
#%% 1.0 Creating class for each asset

# Create a class for returns. Creates a dataframe with returns for every column in the dataset, and creates a dataframe with the same number in %

class Assets:
    def __init__(self,dataset,proxy,columns,proxy_columns,pct_columns):
        self.dataset, self.proxy, self.columns, self.proxy_columns, self.pct_columns = dataset, proxy, columns, proxy_columns,pct_columns
        
    #Function for daily returns
    def returns(self):
        d, p, col, pro_col ,per_col = self.dataset, self.proxy,self.columns,self.proxy.columns, self.pct_columns
        
        #Daily returns
        raw_ret     = pd.DataFrame(data = d.pct_change()[1:],
                                index = d.index,
                                columns = col)
        
        proxy_ret     = pd.DataFrame(data = p.pct_change()[1:],
                                    index=p.index,
                                    columns = pro_col)
        
        #####################################################################################
        # Fills all N/A wih 0 and drops the first observation
        daily_ret    = raw_ret.fillna(value=0).drop(raw_ret.index[0])
        proxy_ret    = proxy_ret.fillna(value=0).drop(proxy_ret.index[0])
        
        
        # Merge proxy med index
        for i in pro_col:
            for x in range(len(proxy_ret)):
                daily_ret[i][x] += proxy_ret[i][x]      
        
        #####################################################################################
        
        #Percentage(%) daily return
        daily_pct_ret = daily_ret.copy()*100        
        daily_pct_ret.columns = per_col
        
        #Cumulative daily return
        cum_daily = pd.DataFrame(data = (1 + daily_ret).cumprod() -1, #cumulative daily return starts
                                 index = d.index,
                                 columns = col).copy()
        
        cum_index = pd.DataFrame(data = (1 + daily_ret).cumprod(), #cumulative daily return starts
                                 index = d.index,
                                 columns = col).copy()
        
        
        return daily_ret, daily_pct_ret, cum_daily, cum_index
    
    #Function for daily returns
    def graph(self):
        cum, cols = self.returns()[2], self.columns
        
        # 1. reset Date as column
        cum = cum.reset_index().copy()
        
        # 2. use 'melt' to go from short- to long format
        df1 = cum.melt(id_vars=['Date'],
                       var_name='ticker',
                       value_name='cum_return',
                       value_vars=cols)
        
        #3. Times 100 to get it in %
        df1['cum_return_pct'] = df1['cum_return'] * 100 # times 100 to get percentage
        # New dataframe created
        
        ## Create plot
        fig = px.line(df1, x='Date',
              y='cum_return_pct', color='ticker',
              title='Performance - Daily Cumulative Returns',
              labels={'cum_return_pct':'daily cumulative returns (%)', },
             width=1000, height=600) 
        fig.update_xaxes(tick0=0, dtick=500)
        fig.update_xaxes(rangeslider_visible=True)
        
        graph = fig.show()
        return graph
    
        
    def summary(self):
        d, col = self.returns()[0], self.columns
        
        summ = pd.DataFrame(data = [ np.round(d.mean()*261*100,2), np.round(d.std()*np.sqrt(261)*100,2), 
                                     np.round(np.var(d)*261*100,2) ],
                    index = ['Annualized Mean', 'Annualized std', 'Annualized Var'],
                    columns = col)
        return summ
#%% 1.1 Create a class for returns who do not use proxy

class noproxy:
    def __init__(self,dataset,columns,pct_columns):
        self.dataset, self.columns, self.pct_columns = dataset, columns, pct_columns
        
    #Function for daily returns
    def returns(self):
        d, col, p_col = self.dataset, self.columns, self.pct_columns
        
        #Daily returns
        daily_ret     = pd.DataFrame(data = d.pct_change()[1:],
                                index = d.index,
                                columns = col)
        
        #Percentage(%) daily return
        daily_pct_ret = daily_ret.copy()*100        
        daily_pct_ret.columns = p_col
        
        #Cumulative daily return
        cum_daily = pd.DataFrame(data = (1 + daily_ret).cumprod() -1, #cumulative daily return starts
                                 index = d.index,
                                 columns = col).copy()
        
        cum_index = pd.DataFrame(data = (1 + daily_ret).cumprod(), #cumulative daily return starts
                                 index = d.index,
                                 columns = col).copy()
        
        
        return daily_ret, daily_pct_ret, cum_daily, cum_index
    
    #Function for daily returns
    def graph(self):
        cum, cols = self.returns()[2], self.columns
        
        # 1. reset Date as column
        cum = cum.reset_index().copy()
        
        # 2. use 'melt' to go from short- to long format
        df1 = cum.melt(id_vars=['Date'],
                       var_name='ticker',
                       value_name='cum_return',
                       value_vars=cols)
        
        #3. Times 100 to get it in %
        df1['cum_return_pct'] = df1['cum_return'] * 100 # times 100 to get percentage
        # New dataframe created
        
        ## Create plot
        fig = px.line(df1, x='Date',
              y='cum_return_pct', color='ticker',
              title='Performance - Daily Cumulative Returns',
              labels={'cum_return_pct':'daily cumulative returns (%)', },
             width=1000, height=600) 
        fig.update_xaxes(tick0=0, dtick=500)
        fig.update_xaxes(rangeslider_visible=True)
        
        graph = fig.show()
        return graph
    
        
    def summary(self):
        d, col = self.returns()[0], self.columns
        
        summ = pd.DataFrame(data = [ np.round(d.mean()*261*100,2), np.round(d.std()*np.sqrt(261)*100,2), 
                                     np.round(np.var(d)*261*100,2) ],
                    index = ['Annualized Mean', 'Annualized std', 'Annualized Var'],
                    columns = col)
        return summ

#%% 1.2 Create a class for excess returns

class excess:
    def __init__(self,dataset,columns,pct_columns, libor):
        self.dataset, self.columns, self.pct_columns, self.libor = dataset, columns, pct_columns, libor
        
    #Function for daily returns
    def returns(self):
        d, col, p_col, libor = self.dataset, self.columns, self.pct_columns, self.libor
        
        #Daily returns
        daily_ret     = pd.DataFrame(data = d.pct_change()[1:],
                                index = d.index,
                                columns = col)
        
        #Convert to excess return
        for i in col:
            for x in range(len(daily_ret)):
                daily_ret[i][x] -= libor['Libor'][x]
        
        #Percentage(%) daily return
        daily_pct_ret = daily_ret.copy()*100        
        daily_pct_ret.columns = p_col
        
        #Cumulative daily return
        cum_daily = pd.DataFrame(data = (1 + daily_ret).cumprod() -1, #cumulative daily return starts
                                 index = d.index,
                                 columns = col).copy()
        
        cum_index = pd.DataFrame(data = (1 + daily_ret).cumprod(), #cumulative daily return starts
                                 index = d.index,
                                 columns = col).copy()
        
        
        return daily_ret, daily_pct_ret, cum_daily, cum_index
    
    #Function for daily returns
    def graph(self):
        cum, cols = self.returns()[2], self.columns
        
        # 1. reset Date as column
        cum = cum.reset_index().copy()
        
        # 2. use 'melt' to go from short- to long format
        df1 = cum.melt(id_vars=['Date'],
                       var_name='ticker',
                       value_name='cum_return',
                       value_vars=cols)
        
        #3. Times 100 to get it in %
        df1['cum_return_pct'] = df1['cum_return'] * 100 # times 100 to get percentage
        # New dataframe created
        
        ## Create plot
        fig = px.line(df1, x='Date',
              y='cum_return_pct', color='ticker',
              title='Performance - Daily Cumulative Returns',
              labels={'cum_return_pct':'daily cumulative returns (%)', },
             width=1000, height=600) 
        fig.update_xaxes(tick0=0, dtick=500)
        fig.update_xaxes(rangeslider_visible=True)
        
        graph = fig.show()
        return graph
    
        
    def summary(self):
        d, col = self.returns()[0], self.columns
        
        summ = pd.DataFrame(data = [ np.round(d.mean()*261*100,2), np.round(d.std()*np.sqrt(261)*100,2), 
                                     np.round(np.var(d)*261*100,2) ],
                    index = ['Annualized Mean', 'Annualized std', 'Annualized Var'],
                    columns = col)
        return summ
#%% 2.0 Organizing data
# 2.1 Creating new columns for the classes

#Equity
eq_cols.pop(0)
eq_pct_cols = ['FTSE %','S&P 500 %', 'IBEX 35 %', 'AEX %', 'TOPIX %', 'MIB %', 'DAX %', 'CAC 40 %', 'SPI 200 %']

#Bond
bond_cols.pop(0)
bond_pct_cols   = ['3-year AUS%','10-year AUS%','2-year EURO%','5-year EURO%','10-year EURO%','30-year EURO%','10-year JP%','10-year CAN%','10-year UK%','2-year US%','5-year US%','10-year US%','30-year US%']

#Commodities
comm_cols.pop(0)
comm_pct_cols = ['Platinum%', 'Silver%', 'Gold%','Heating Oil%','Natural Gas%','Unleaded%','WTI Crude%','Wheat%','Soy Oil%','Soy Meal%','Soybean%','Corn%','Cattle%','Lean Hogs%','Gas Oil%','Brent Crude%','Coffee%','Sugar%','Cotton%','Cocoa%','Zinc%','Nickel%','Copper%','Aluminium%']

#FX
fx_cols.pop(0)
fx_pct_cols = ['AUD-USD%','EUR-USD%','CAD-USD%','JPY-USD%','NOK-USD%','NZD-USD%','CHF-USD%','GBP-USD%','SEK-USD%']

#Proxy
proxyE_cols = ['IBEX 35', 'TOPIX', 'MIB', 'DAX','CAC 40', 'SPI 200']
proxyE_pct_cols = ['IBEX 35 %', 'TOPIX %', 'MIB %', 'DAX %','CAC 40 %', 'SPI 200 %']

proxyB_cols = ['AUS 3-year','AUS 10-year','EURO SCHATZ','EURO BOBL','EURO BUND','EURO BUXL', 'CAN 10-year','TSE 10-year','LONG GILT','US 2-year', 'US 10-year','US 30-year']
proxyB_pct_cols = ['AUS 3-year %','AUS 10-year %','EURO SCHATZ %','EURO BOBL %','EURO BUND %','EURO BUXL %', 'CAN 10-year %','TSE 10-year %','LONG GILT %','US 2-year %', 'US 10-year %','US 30-year %']

new_proxyFX_cols = ['AUD-USD','EUR-USD','CAD-USD','JPY-USD','NOK-USD','NZD-USD','CHF-USD','GBP-USD','SEK-USD']
proxyFX_pct_cols = ['AUD-USD%','EUR-USD%','CAD-USD%','JPY-USD%','NOK-USD%','NZD-USD%','CHF-USD%','GBP-USD%','SEK-USD%']

#%% 2.1 Initiate assets in class

equity = Assets(equity_raw, proxyE_raw, eq_cols, proxyE_cols ,eq_pct_cols)
bond   = noproxy(bond_raw, bond_cols, bond_pct_cols)
commod = noproxy(comm_raw, comm_cols, comm_pct_cols)
fx     = noproxy(fx_raw,   fx_cols,   fx_pct_cols)

#fx2    = excess(fx_raw, fx_cols, fx_pct_cols, libor)

#%% 2.2 Compute daily data

### 3.0 Returns
## 3.1 Daily return in decimals
# Retrieve daily returns in decimals
eq_daily_ret   = equity.returns()[0]
bond_daily_ret = bond.returns()[0]
comm_daily_ret = commod.returns()[0]
fx_daily_ret   = fx.returns()[0]

#fx2_daily_ret  = fx2.returns()[0]

## 3.2 Daily return in percentages
#Equity
eq_pct_ret   = equity.returns()[1]
bond_pct_ret = bond.returns()[1]
comm_pct_ret = commod.returns()[1]
fx_pct_ret   = fx.returns()[1]

#fx2_pct_ret   = fx2.returns()[1]

# 3.2 Daily cumulative return
eq_cum_daily   = equity.returns()[2]
bond_cum_daily = bond.returns()[2]
comm_cum_daily = commod.returns()[2]
fx_cum_daily   = fx.returns()[2]

#fx2_cum_daily   = fx2.returns()[2]

# 3.2 Daily cumulative index (daily)
eq_cum_index   = equity.returns()[3]
bond_cum_index = bond.returns()[3]
comm_cum_index = commod.returns()[3]
fx_cum_index   = fx.returns()[3]

#fx2_cum_index  = fx2.returns()[3]

#%% 2.3 Summary statistics
# Mean & Variance
# STD and Var in percentages

eq_summary   = equity.summary()
comm_summary = commod.summary()
bond_summary = bond.summary()
fx_summary   = fx.summary()

#fx2_summary  = fx2.summary()
#%% 3.0 Scale by volatility
# 3.1 Constructing an average return for each point in time

class scaling:
    def __init__(self, daily_ret, data, columns):
        self.daily_return, self.data, self.columns = daily_ret, data, columns
        
    #Function for variance with centre of mass = 60
    def com(self):
        daily_ret, cols = self.daily_return, self.columns
        
        mov_var = daily_ret.copy()
        
        for cols in mov_var:
            mov_var[cols] = 261 * mov_var[cols].ewm(com=60, ignore_na=True, adjust=False).var()
        
        return mov_var
# https://towardsdatascience.com/trading-toolbox-02-wma-ema-62c22205e2a9
# https://tedboy.github.io/pandas/computation/computation5.html
# https://stackoverflow.com/questions/45059676/python-pandas-calculating-exponentially-weighted-lagged-squared-returns-varian


# Funksjon for å omgjøre rådataen/variansen fra daglig til månedlig
    def monthly_return(self):
        data = self.data
        
        data.index = pd.to_datetime(data.index)
        monthly_ret = data.resample('M').ffill().pct_change().copy()
    
        return monthly_ret
    
# Trenger data fra 31.12.1984. Kan bare kopiere pris fra 01.01.1985
    def monthly_variance(self):
        var = self.com()
        var.index = pd.to_datetime(var.index)
        monthly_var = var.resample('M').ffill().copy()
    
        return monthly_var
# Double checked, extracts the last value of the month. Since it is annualized, this is fine
# https://www.codingfinance.com/post/2018-04-03-calc-returns-py/

    def monthly_cum(self):
        ret = self.monthly_return()
        
        cum_monthly = (1 + ret.copy()).cumprod() -1
        
        return cum_monthly

# Function for scaling on ex ante volatility, and weights
    def scaled(self):
        monthly_ret, monthly_var = self.monthly_return(), self.monthly_variance()
    
    #Use shift 1 to use the previous month variance, since it's lagged
        monthly_lagged = monthly_var.shift(1)
    
    #Find the weight; the inverse lagged volatility
        weight = 1 / np.sqrt(monthly_lagged)
    
    ## Divide the monthly return on the volatility, not variance
        scaled_ret = monthly_ret * weight
    
        return scaled_ret, weight

# Question: Should we use the last observation of the month, or calculate monthly?       

#%% 3.1 Monthly return, variance, weight, and scaled return

#Initiating the data in the classes
e22 = scaling(eq_daily_ret, eq_cum_index, eq_cols)
b22 = scaling(bond_daily_ret, bond_cum_index, bond_cols)
c22 = scaling(comm_daily_ret, comm_cum_index, comm_cols)
f22 = scaling(fx_daily_ret, fx_cum_index, fx_cols)

A = e22.monthly_cum()
#Equity
eq_monthly_ret, eq_monthly_var = e22.monthly_return(), e22.monthly_variance()
eq_monthly_std = np.sqrt(eq_monthly_var)

eq_scaled_ret, eq_weight = e22.scaled()[0], e22.scaled()[1]

#Bond
bond_monthly_ret, bond_monthly_var = b22.monthly_return(), b22.monthly_variance()
bond_monthly_std = np.sqrt(bond_monthly_var)

bond_scaled_ret, bond_weight = b22.scaled()[0], b22.scaled()[1]

#Commodity
comm_monthly_ret, comm_monthly_var = c22.monthly_return(), c22.monthly_variance()
comm_monthly_std = np.sqrt(comm_monthly_var)

comm_scaled_ret, comm_weight = c22.scaled()[0], c22.scaled()[1]

#FX
fx_monthly_ret, fx_monthly_var = f22.monthly_return(), f22.monthly_variance()
fx_monthly_std = np.sqrt(fx_monthly_var)

fx_scaled_ret, fx_weight = f22.scaled()[0], f22.scaled()[1]

del bond_daily_ret
del comm_daily_ret
del eq_daily_ret
del fx_daily_ret


#%% 4.0 Function for pooled regression
# Update 
# We have now scaled all returns on its previous months volatility, which means both the current return, and holding period return is scaled on the lagged volatility. Now we can regress
# 5.3 Pool regressions of instruments
# Panel A: Pooled regression by month h lag

from linearmodels import PooledOLS

#Creating dataframe with t-statistics for each holding period

def tstats(scaled_returns):
    
    tt = []
    
    for i in range(60):
    
        y = scaled_returns
        x = scaled_returns.shift(i+1)
    
        mod = PooledOLS(y,x)
        res = mod.fit(cov_type='clustered', cluster_entity=True)
        tt.append(res.tstats[0])
        transposed = np.array(tt).T
        
    tstats = pd.DataFrame(data = transposed ,
                           index = [ list(range(1,61)) ],
                           columns = ['T_stat'])
    return tstats


#%% 4.1 Graphics: Panel A Individual
#Equity Graph
eq_tstat = tstats(eq_scaled_ret)

fig_eq = eq_tstat.plot.bar(x=None, y=None, xticks = range(0,60,3), yticks = range(-2,3,1),legend=None, \
                           title='T-statistics by month, Equity', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.")


# Cut the NAN in bonds and commodity for pooled data
comm_pooled = comm_scaled_ret.copy()
comm_pooled.dropna(inplace=True)

bond_pooled = bond_scaled_ret.copy()
bond_pooled.dropna(inplace=True)

#Bond Graph
bond_tstat = tstats(bond_pooled)

fig_bond = bond_tstat.plot.bar(x=None, y=None, xticks = range(0,60,3), legend=None, \
                         title='T-statistics by month, Bonds', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.") 

#Commodity Graph
comm_tstat = tstats(comm_pooled)

fig_comm = comm_tstat.plot.bar(x=None, y=None, xticks = range(0,60,3), legend=None, \
                         title='T-statistics by month, Commodity', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.") 
    
#FX Graph
fx_tstats = tstats(fx_scaled_ret)

fig_fx = fx_tstats.plot.bar(x=None, y=None, xticks = range(0,60,3), legend=None, \
                         title='T-statistics by month, FX', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.")


#%% 4.2 Panel A all assets

#Change length of datasets
#Equity return
eq_pooledA = eq_scaled_ret.copy()
eq_pooledA.drop(eq_pooledA.index[0:(len(eq_scaled_ret)-len(bond_pooled))],0, inplace=True) #444
#Commodity return
comm_pooledA = comm_pooled.copy()
comm_pooledA.drop(comm_pooledA.index[0:(len(comm_pooled)-len(bond_pooled))],0, inplace=True) #380
#FX return
fx_pooledA = fx_scaled_ret.copy()
fx_pooledA.drop(fx_pooledA.index[0:(len(fx_scaled_ret)-len(bond_pooled))],0, inplace=True) #444

# Merge dataframes
pooled_A = pd.concat([eq_pooledA,bond_pooled,comm_pooledA,fx_pooledA], axis=1)

#Initiate in function
panel_A = tstats(pooled_A)

#Create figure
fig_A = panel_A.plot.bar(x=None, y=None, xticks = range(0,60,3), legend=None, \
                         title='T-statistics by month, all assets', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.") 

#%% 5.0 TSMOM factor Class

class TSMOM_F:
    def __init__(self,monthly_ret, cols, weights, months):
        self.monthly_ret, self.cols, self.weights, self.months = monthly_ret, cols, weights, months
        
        
    def TSMOM(self):
        month_ret, cols, weights, m = self.monthly_ret, self.cols, self.weights, self.months
        
        #Log returns for easier calculations for the sign
        log_ret = np.log(month_ret.copy() +1)
        log_ret.drop(log_ret.index[0],0, inplace=True)
        
        #Check the cum return last "m" months
        prior_n = log_ret.rolling(m).sum().copy()
        prior_n.drop(prior_n.index[0:11],0, inplace=True) #Not 12, so we can shift it to ex ante the first month
        
        #Create a DF for position (+1/-1)
        position = prior_n.copy()
        
        for x in range(len(cols)):
            for a in range(len(position)):
                if   prior_n[cols[x] ] [a] > 0:
                     position[cols[x]] [a] = 1
                elif prior_n[cols[x] ] [a] < 0:
                     position[cols[x]] [a] = -1
               
        #################################################################
        #Create dataframe for strategy
        ret = month_ret.copy()
        ret.drop(ret.index[0:(len(month_ret)-len(position))],0,inplace=True)
        
        strat = position.shift(1) * ret
        strat.drop(strat.index[0],0, inplace=True)
    
        #Clear the length of the weights
        w = weights.copy()
        w.drop(w.index[0:(len(w)-len(strat))],0, inplace=True)
        weights_scaled = 0.4 * w
        
        #Clear the length of simple returns
        ret.drop(ret.index[0:(len(ret)-len(strat))],0, inplace=True) #Endret!
        
        #Scaling the strategy
        strat_scaled = strat.copy()
        strat_scaled = strat * weights_scaled
        
        #Cumulative of the returns
        cum_scaled = pd.DataFrame(data = (1 + strat_scaled).cumprod() -1, #cumulative daily return starts
                                 index = strat_scaled.index,
                                 columns = cols).copy()

        
        return position, strat, strat_scaled, cum_scaled, weights_scaled
    
#%% 5.1 TSMOM_F for asset classes

#Equities
e33 = TSMOM_F(eq_monthly_ret, eq_cols, eq_weight, 12)

tsmom_e_pos          = e33.TSMOM()[0]
tsmom_e_strat        = e33.TSMOM()[1]
tsmom_e_strat_scaled = e33.TSMOM()[2]
tsmom_e_cum_strat    = e33.TSMOM()[3]

#Bonds
b33 = TSMOM_F(bond_monthly_ret, bond_cols, bond_weight, 12)

tsmom_b_pos          = b33.TSMOM()[0]
tsmom_b_strat        = b33.TSMOM()[1]
tsmom_b_strat_scaled = b33.TSMOM()[2]
tsmom_b_cum_strat    = b33.TSMOM()[3]

#Commodities
c33 = TSMOM_F(comm_monthly_ret, comm_cols, comm_weight, 12)

tsmom_c_pos          = c33.TSMOM()[0]
tsmom_c_strat        = c33.TSMOM()[1]
tsmom_c_strat_scaled = c33.TSMOM()[2]
tsmom_c_cum_strat    = c33.TSMOM()[3]

#FX
f33 = TSMOM_F(fx_monthly_ret, fx_cols, fx_weight, 12)

tsmom_f_pos          = f33.TSMOM()[0]
tsmom_f_strat        = f33.TSMOM()[1]
tsmom_f_strat_scaled = f33.TSMOM()[2]
tsmom_f_cum_strat    = f33.TSMOM()[3]


#%% 6.0 Panel B: Pooled regression with standard errors clustered monthly, looking at previous sign

def t_abs(scaled_returns, sign):
    
    #s_ret = scaled_returns.copy()
    #s_ret.drop(s_ret.index[0:12],0, inplace=True)
    
    tt = []
    
    for i in range(60):
    
        y = scaled_returns
        x = sign.shift(i+1)
        #x = returns.shift(i+1)
    
        mod = PooledOLS(y,x)
        res = mod.fit(cov_type='clustered', cluster_entity=True)
        tt.append(res.tstats[0])
        transposed = np.array(tt).T
        
#https://bashtage.github.io/linearmodels/panel/panel/linearmodels.panel.results.PanelResults.html#linearmodels.panel.results.PanelResults
        
    t_abs = pd.DataFrame(data = transposed ,
                           index = [ list(range(1,61)) ],
                           columns = ['T_stat'])
    return t_abs


#%% 6.1 Print out Panel C
#
#Lik lengde på ret og sign
eq_scaled_ret_B = eq_scaled_ret.copy()
eq_scaled_ret_B.drop(eq_scaled_ret_B.index[0:12],0, inplace=True)

tsmom_b_pos_B    = tsmom_b_pos.copy()
tsmom_b_pos_B.drop(tsmom_b_pos_B.index[0:154],0, inplace=True)

tsmom_c_pos_B  = tsmom_c_pos.copy()
tsmom_c_pos_B.drop(tsmom_c_pos_B.index[0:52],0, inplace=True)

fx_scaled_ret_B = fx_scaled_ret.copy()
fx_scaled_ret_B.drop(fx_scaled_ret_B.index[0:12],0, inplace=True)

#Fix length of datasets
eq_tstat2   = t_abs(eq_scaled_ret_B, tsmom_e_pos)  # 444 vs 432

bond_tstat2 = t_abs(bond_pooled, tsmom_b_pos_B)    # 278 vs 432

comm_tstat2 = t_abs(comm_pooled, tsmom_c_pos_B)    # 380 vs 432

fx_tstat2   = t_abs(fx_scaled_ret_B, tsmom_f_pos)  # 444 vs 432

#Graphs: Panel C

#Equity Graph
fig_eq2 = eq_tstat2.plot.bar(x=None, y=None, xticks = range(0,60,3), legend=None, \
                         title='T-statistics by month, Equity', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.")
   
#Bond Graph
fig_bond2 = bond_tstat2.plot.bar(x=None, y=None, xticks = range(0,60,3),yticks = range(-1,3,1), legend=None, \
                         title='T-statistics by month, Bonds', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.")

#Comm Graph
fig_comm2 = comm_tstat2.plot.bar(x=None, y=None, xticks = range(0,60,3),yticks = range(-3,3,1), legend=None, \
                         title='T-statistics by month, Commodity', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.")

#FX Graph
fig_fx2 = fx_tstat2.plot.bar(x=None, y=None, xticks = range(0,60,3), legend=None, \
                         title='T-statistics by month, FX', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.")
    
    
#%% 6.2 Print out Panel B
#Fix datalength: Alle length of 278
#Equity return
eq_pooledB = eq_pooledA.copy()
#eq_pooledB.drop(eq_pooledB.index[0:(len(eq_scaled_ret)-len(bond_pooled))],0, inplace=True) #444
#Equity sign
eq_pooled_s = tsmom_e_pos.copy()
eq_pooled_s.drop(eq_pooled_s.index[0:(len(tsmom_e_pos)-len(bond_pooled))],0,inplace=True)
#Commodity return
comm_pooledB = comm_pooledA.copy()
#comm_pooledB.drop(comm_pooledB.index[0:(len(comm_pooled)-len(bond_pooled))],0, inplace=True) #380
#Commodity sign
comm_pooled_s = tsmom_c_pos_B.copy()
comm_pooled_s.drop(comm_pooled_s.index[0:(len(comm_pooled_s)-len(bond_pooled))],0,inplace=True)
#FX return
fx_pooledB = fx_pooledA.copy()
#fx_pooledB.drop(fx_pooledB.index[0:(len(fx_scaled_ret)-len(bond_pooled))],0, inplace=True) #444
#FX sign
fx_pooled_s = tsmom_f_pos.copy()
fx_pooled_s.drop(fx_pooled_s.index[0:(len(fx_pooled_s)-len(bond_pooled))],0,inplace=True)

# Merge dataframes
pooled_c    = pd.concat([eq_pooledB,bond_pooled,comm_pooledB,fx_pooledB], axis=1)
pooled_sign = pd.concat([eq_pooled_s, tsmom_b_pos_B,comm_pooled_s,fx_pooled_s], axis=1)


#Initiate the regression
panel_B = t_abs(pooled_c, pooled_sign)

#Print Panel B
fig_eq2 = panel_B.plot.bar(x=None, y=None, xticks = range(0,60,3), legend=None, \
                         title='T-statistics by month, all assets', figsize=(7,4.5)).grid(axis='y', color ='grey', linewidth = "0.5", linestyle = "-.")


#%% 7.0 Equally weighted portfolio simple returns, all assets classes
# Function to create DF with return and cumulative return

def EW(monthly_ret, cols):
    sec = monthly_ret.copy()
    sec['Number'] = sec.count(axis=1)
    sec.drop(sec.index[0],0, inplace=True)
    
    for i in cols:
        del sec[i]
    
    eqw = monthly_ret.copy()
    eqw.drop(eqw.index[0],0, inplace=True)
    for i in cols:
        for x in range(len(eqw)):
            eqw[i][x] /= sec['Number'][x]
    
    
    sum = monthly_ret.copy()
    sum['Sum'] = eqw.sum(axis=1)
    sum.drop(sum.index[0],0, inplace=True)
    
    for i in cols:
        del sum[i]
        
    sum['Cum'] = (1 + sum['Sum']).cumprod() -1
        
    return sec, sum
        
#%% 7.1 Initiate and graph the portfolios

eq_ew    = EW(eq_monthly_ret, eq_cols)[1]
bond_ew  = EW(bond_monthly_ret, bond_cols)[1]
comm_ew  = EW(comm_monthly_ret, comm_cols)[1]
fx_ew    = EW(fx_monthly_ret, fx_cols)[1]

#Remove the return columns
del eq_ew['Sum']
del bond_ew['Sum']
del comm_ew['Sum']
del fx_ew['Sum']

# Merge the cumulative returns
eqw_cum = pd.concat([eq_ew, bond_ew, comm_ew, fx_ew], axis=1)
eqw_cum.columns = ['Equity', 'Bonds', 'Commodity', 'FX']

#Create the figure
fig_eqw_cum = eqw_cum.plot.line(x=None, y=None, \
                         title='Cumulative return EW portfolios', figsize=(18,10)).grid(axis='y', color ='grey')
    

#%% 8.0 Cumulative return Long only vs TSOM
# Function to calculate number of securities

def EW2(strat, cols):
    sec = strat.copy()
    sec['Number'] = sec.count(axis=1)
    
    for i in cols:
        del sec[i]
    
    eqw = strat.copy()
    for i in cols:
        for x in range(len(eqw)):
            eqw[i][x] /= sec['Number'][x]
    
    
    sum = strat.copy()
    sum['Sum'] = eqw.sum(axis=1)
    
    for i in cols:
        del sum[i]
        
    sum['Cum']   = (1 + sum['Sum']).cumprod() -1
    
    index = sum.copy()
    index['Index'] = (1 + sum['Sum']).cumprod()
    del index['Sum']
    del index['Cum']
    
    return sec, sum, index, eqw

#%% 8.1: Initiate dataframes for passive vs active
# Long only
long = pd.concat([eq_monthly_ret, bond_monthly_ret, comm_monthly_ret, fx_monthly_ret], axis=1)
long.drop(long.index[0:(len(long)-len(tsmom_e_strat_scaled))],0,inplace=True)
all_cols = eq_cols + bond_cols + comm_cols + fx_cols
A2 = EW2(long, all_cols)[1]

# Strategy with long/short
tsmom_strat = pd.concat([tsmom_e_strat_scaled, tsmom_b_strat_scaled, tsmom_c_strat_scaled, tsmom_f_strat_scaled], axis=1)
A22 = EW2(tsmom_strat, all_cols)[1]

#Merge the cum returns into one dataframe
A23 = pd.concat([A2['Cum'], A22['Cum']], axis=1)
A23.columns = ['Passive long strategy', 'TSMOM']


#Create the figure
fig_strat_cum = A23.plot.line(x=None, y=None, \
                         title='Cumulative return Long vs Strategy', figsize=(18,10)).grid(axis='y', color ='grey')               
        
#%% 8.2 Sharpe Ratios for TSMOM

#Equity
def sharpe(tsmom, cols):
    ret    = (1+tsmom.mean())**12 -1
    std    = tsmom.std()*np.sqrt(12)
    sharpe = ret / std
    test   = (ret / (std / np.sqrt(len(tsmom)) ) )
    
    summary = pd.DataFrame( data = [np.round(ret,2), np.round(std,2), np.round(sharpe,2), np.round(test,2) ],
                           index = ['Annualized Mean', 'Annualized std', 'Annualized Sharpe Ratio', 'Mean Test'],
                           columns = cols)
    
    return summary
        
        
sharpe_tsmom_e = sharpe(tsmom_e_strat_scaled, eq_cols)
sharpe_tsmom_b = sharpe(tsmom_b_strat_scaled, bond_cols)
sharpe_tsmom_c = sharpe(tsmom_c_strat_scaled, comm_cols)
sharpe_tsmom_f = sharpe(tsmom_f_strat_scaled, fx_cols)
sharpe_tsmom   = sharpe(A22['Sum'], ['Sum'])
sharpe_passive = sharpe(A2['Sum'], ['Sum'])


#%% 9.0 Alfa: Make benchmarks monthly return

#Barclays: 2 datasets of monthly(start 1989) and daily prices, need to compute return, and then merge to daily
fnam_barc1      = "Barclays_daily.csv"
fnam_barc2      = "Barclays_mnd.csv"

barc1_raw   = pd.read_csv(fnam_barc1, delimiter=';', decimal=',')
barc2_raw   = pd.read_csv(fnam_barc2, delimiter=';', decimal=',')

barc_cols    = ['Date','Barclays Bond']
barc1_raw.columns = barc_cols
barc2_raw.columns = barc_cols
barc_cols.pop(0)

barc1_raw.set_index('Date',inplace=True)
barc2_raw.set_index('Date',inplace=True)

barc1_raw.index = pd.to_datetime(barc1_raw.index)
barc2_raw.index = pd.to_datetime(barc2_raw.index)

#Change barc1 to monthly data, and calculate return for both
barc1_month_ret = barc1_raw.resample('M').ffill().pct_change().copy()

barc2_month_ret    = pd.DataFrame(data = barc2_raw.pct_change()[1:],
                                  index = barc2_raw.index,
                                  columns = barc_cols)

#Drop the first obsercation and put NA to 0
barc1_month_ret = barc1_month_ret.fillna(value=0).drop(barc1_month_ret.index[0])
barc2_month_ret = barc2_month_ret.fillna(value=0).drop(barc2_month_ret.index[0])
        
#Create a new dataframe and merge the returns
barc_month_ret = barc1_month_ret.copy()
for x in range(len(barc2_month_ret)):
      barc_month_ret['Barclays Bond'][x] = barc1_month_ret['Barclays Bond'][x] + barc2_month_ret['Barclays Bond'][x] 

#Removing unnecessairy variables
del fnam_barc1
del fnam_barc2
del barc1_month_ret
del barc1_raw
del barc2_month_ret
del barc2_raw

#Load the risk free rate
fnam_rf = "risk_free.csv"
rf      = pd.read_csv(fnam_rf, delimiter=';', decimal='.')
rf.set_index('Date',inplace=True)
rf = rf.drop(rf.index[0])
rf.index = pd.to_datetime(rf.index)

#Make Barclays the same length as risk free and carhart, and subtracting risk free to get excess
barc_month_ret.drop(barc_month_ret.tail( len(barc_month_ret)-len(rf) ).index, inplace=True )

for x in range(len(barc_month_ret)):
    barc_month_ret['Barclays Bond'][x] -= rf['RF'][x]
    
#Carhart already loaded, getting benchmark: 
fnam_bench = "Benchmark.csv"
bench      = pd.read_csv(fnam_bench, delimiter=';', decimal=',')
bench_cols = ['Date','MSCI', 'GSCI']
bench.columns = bench_cols
bench_cols.pop(0)
bench.set_index('Date',inplace=True)

#Calculate monthly return
bench.index = pd.to_datetime(bench.index)
bench_month_ret = bench.resample('M').ffill().pct_change().copy()
bench_month_ret = bench_month_ret.drop(bench_month_ret.index[0])


#Make benchmark length as risk free and carhart
bench_month_ret.drop(bench_month_ret.tail( len(bench_month_ret)-len(rf) ).index, inplace=True )

del bench

#Subtract risk-free from MSCI and GSCI
for i in bench_cols:
    for x in range(len(bench_month_ret)):
        bench_month_ret[i][x] -= rf['RF'][x]

#%% 9.1 Same length of Y and X
# Make a dataframe for return on 
tsmom_ret = A22.copy()
del tsmom_ret['Cum']

#Making a copy for later
tsmom_ret_full = tsmom_ret.copy()

all_bench = pd.concat([bench_month_ret['MSCI'], barc_month_ret['Barclays Bond'],bench_month_ret['GSCI'], carhart_ret ], axis=1)
#Benchmarks start 31-01-1985, TSOM starts 28-02-1986
all_bench.drop(all_bench.index[0:13],0,inplace=True)

#Benchmarks ends at 31-05-2019, TSMOM ends at 31-12-2009
all_bench.drop(all_bench.tail( len(all_bench)-len(tsmom_ret) ).index, inplace=True )


#%% 9.2 Alpha: Regression and results

Y = tsmom_ret
X = all_bench
X = sm.add_constant(X)
model       = sm.OLS(Y,X)
results = model.fit()
print(results.params)
#%% 9.3 Alpha: regression and result, fewer benchmarks

Y2 = tsmom_ret.copy()
X2 = X.copy() 

del X2['Barclays Bond']
del X2['GSCI']

model2 = sm.OLS(Y2, X2)
results2 = model2.fit()

#%% 9.4 Alpha: Regression with only significant benchmarks

Y3 = tsmom_ret.copy()
X3 = X.copy()

del X3['SMB']
del X3['HML']
del X3['MSCI']
del X3['GSCI']
model3   = sm.OLS(Y3, X3)
results3 = model3.fit()


#%% 10.0 Quarterly S&P 500 vs TSMOM
#1: Make TSMOM returns quarterly

#Create price index
tsmom_index = EW2(tsmom_strat, all_cols)[2]

tsmom_index.loc[tsmom_c_pos.first_valid_index()] = [1]  # adding a row
tsmom_index = tsmom_index.sort_index() #Sorting the index by date

#Calculate quarterly returns
def quart_return(index):
        
        quart_ret = index.resample('Q').ffill().pct_change().copy()
        quart_ret.drop(quart_ret.index[0],0, inplace=True)
        
        return quart_ret
    
tsmom_quart = quart_return(tsmom_index)

#%% 10.0 Figure 4: S&P vs TSMOM quarterly returns
sp500 = equity_raw.copy()
sp500.index = pd.to_datetime(sp500.index)

del sp500['CAC 40']
del sp500['FTSE']
del sp500['IBEX 35']
del sp500['AEX']
del sp500['TOPIX']
del sp500['MIB']
del sp500['DAX']
del sp500['SPI 200']

sp500_quart = quart_return(sp500)

#S&P starts before tsmom, cut the dataset
sp500_quart.drop(sp500_quart.index[0:(len(sp500_quart)-len(tsmom_quart))],0,inplace=True)
plt.scatter(sp500_quart, tsmom_quart,  label = 'S&P 500 vs TSMOM')

#tsmom_quart.to_excel(r'C:\Users\hamme\OneDrive\Dokumenter\BI\Master Thesis\Python\Trial1\tsmom_quart.xlsx', index = True)
#sp500_quart.to_excel(r'C:\Users\hamme\OneDrive\Dokumenter\BI\Master Thesis\Python\Trial1\sp500_quart.xlsx', index = True)

#%% 11.0 Correlation within Asset class

# 1. Pair-wise correlation within asset class: TSMOM
corr_tsmom_eq   = tsmom_e_strat_scaled.corr()
corr_tsmom_bond = tsmom_b_strat_scaled.corr()
corr_tsmom_comm = tsmom_c_strat_scaled.corr()
corr_tsmom_fx   = tsmom_f_strat_scaled.corr()


# Sum of the correlation matrix, need to divide it by number of observations
def sum_corr(matrix, cols):
    
    sum = ((matrix / 2).sum(axis=0) ).sum() - (len(cols)/2)
    
    return sum

corr_tsmom_eq2   = sum_corr(corr_tsmom_eq, eq_cols)       / 36
corr_tsmom_bond2 = sum_corr(corr_tsmom_bond, bond_cols)   / 91
corr_tsmom_comm2 = sum_corr(corr_tsmom_comm, comm_cols)   / 276
corr_tsmom_fx2   = sum_corr(corr_tsmom_fx, fx_cols)       / 78

# 2. Passive long
corr_eq   = eq_monthly_ret.corr()
corr_bond = bond_monthly_ret.corr()
corr_comm = comm_monthly_ret.corr()
corr_fx   = fx_monthly_ret.corr()

# 2.1 Average:
corr_eq   = sum_corr(corr_eq, eq_cols)       / 36
corr_bond = sum_corr(corr_bond, bond_cols)   / 91
corr_comm = sum_corr(corr_comm, comm_cols)   / 276
corr_fx   = sum_corr(corr_fx, fx_cols)       / 78

#%% 11.1 Correlation beetween asset class

#Passive
ew_eq    = EW(eq_monthly_ret, eq_cols)[1]
ew_bond  = EW(bond_monthly_ret, bond_cols)[1]
ew_comm  = EW(comm_monthly_ret, comm_cols)[1]
ew_fx    = EW(fx_monthly_ret, fx_cols)[1]

#Remove the return columns
del ew_eq['Cum']
del ew_bond['Cum']
del ew_comm['Cum']
del ew_fx['Cum']

pass_eq   = [ ew_eq['Sum'].corr(ew_eq['Sum']),   ew_eq['Sum'].corr(ew_bond['Sum']),   ew_eq['Sum'].corr(ew_comm['Sum']),   ew_eq['Sum'].corr(ew_fx['Sum'])    ]
pass_bond = [ ew_bond['Sum'].corr(ew_eq['Sum']), ew_bond['Sum'].corr(ew_bond['Sum']), ew_bond['Sum'].corr(ew_comm['Sum']), ew_bond['Sum'].corr(ew_fx['Sum'])  ]
pass_comm = [ ew_comm['Sum'].corr(ew_eq['Sum']), ew_comm['Sum'].corr(ew_bond['Sum']), ew_comm['Sum'].corr(ew_comm['Sum']), ew_comm['Sum'].corr(ew_fx['Sum'])  ]
pass_fx   = [ ew_fx['Sum'].corr(ew_eq['Sum']),   ew_fx['Sum'].corr(ew_bond['Sum']),   ew_fx['Sum'].corr(ew_comm['Sum']),   ew_fx['Sum'].corr(ew_fx['Sum'])    ]

pass_eq   = pd.DataFrame(data = pass_eq ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['Equity'])
pass_bond = pd.DataFrame(data = pass_bond ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['Bond'])
pass_comm = pd.DataFrame(data = pass_comm ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['Commodity'])
pass_fx   = pd.DataFrame(data = pass_fx ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['FX'])

## TSMOM
tsmom_e_ew = EW2(tsmom_e_strat_scaled, eq_cols)[1]
del tsmom_e_ew['Cum']

tsmom_b_ew = EW2(tsmom_b_strat_scaled, bond_cols)[1]
del tsmom_b_ew['Cum']

tsmom_c_ew = EW2(tsmom_c_strat_scaled, comm_cols)[1]
del tsmom_c_ew['Cum']

tsmom_f_ew = EW2(tsmom_f_strat_scaled, fx_cols)[1]
del tsmom_f_ew['Cum']

#tsmom_e_strat_scaled, tsmom_b_strat_scaled, tsmom_c_strat_scaled, tsmom_f_strat_scaled

corr_tsmom_eq   = [ tsmom_e_ew['Sum'].corr(tsmom_e_ew['Sum']), tsmom_e_ew['Sum'].corr(tsmom_b_ew['Sum']), tsmom_e_ew['Sum'].corr(tsmom_c_ew['Sum']), tsmom_e_ew['Sum'].corr(tsmom_f_ew['Sum'])    ]
corr_tsmom_bond = [ tsmom_b_ew['Sum'].corr(tsmom_e_ew['Sum']), tsmom_b_ew['Sum'].corr(tsmom_b_ew['Sum']), tsmom_b_ew['Sum'].corr(tsmom_c_ew['Sum']), tsmom_b_ew['Sum'].corr(tsmom_f_ew['Sum'])    ]
corr_tsmom_comm = [ tsmom_c_ew['Sum'].corr(tsmom_e_ew['Sum']), tsmom_c_ew['Sum'].corr(tsmom_b_ew['Sum']), tsmom_c_ew['Sum'].corr(tsmom_c_ew['Sum']), tsmom_c_ew['Sum'].corr(tsmom_f_ew['Sum'])    ]
corr_tsmom_fx   = [ tsmom_f_ew['Sum'].corr(tsmom_e_ew['Sum']), tsmom_f_ew['Sum'].corr(tsmom_b_ew['Sum']), tsmom_f_ew['Sum'].corr(tsmom_c_ew['Sum']), tsmom_f_ew['Sum'].corr(tsmom_f_ew['Sum'])    ]


corr_tsmom_eq   = pd.DataFrame(data = corr_tsmom_eq ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['Equity'])
corr_tsmom_bond = pd.DataFrame(data = corr_tsmom_bond ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['Bond'])
corr_tsmom_comm = pd.DataFrame(data = corr_tsmom_comm ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['Commodity'])
corr_tsmom_fx   = pd.DataFrame(data = corr_tsmom_fx ,
                           index = ['Equity','Bond', 'Commodity', 'FX'],
                           columns = ['FX'])

#%% 12.0 Drawdown control: Maximum Drawdown

V  = EW2(tsmom_strat, all_cols)[2]
DD = pd.DataFrame(np.zeros([V.shape[0], V.shape[1]]))

for i in range (0, V.shape[1]):
    maxV = V.iloc[1,i]
    for t in range(2, V.shape[0]):
        DD.iloc[t,i] = min(1.0, V.iloc[t,i]/maxV) - 1
        maxV = max(V.iloc[t,i], maxV)

max_dd = pd.DataFrame({'TSMOM': DD.iloc[:,0].min()}, index=[0])

#%% 12.1 Drawdown control: Plot of drawdowns

labels       = max_dd.columns
V            = V.reset_index()
DDplot       = pd.concat([V.iloc[:,0],DD], axis=1)
DDplot.index = pd.to_datetime(DDplot.Date, format="%Y%m")
DDplot       = DDplot.drop(['Date'], axis=1)
DDplot.columns = ['Drawdown']

fig, ax = plt.subplots(figsize=(10,6), dpi=100)

for i in range(0,DDplot.shape[1]):
    color=next(ax._get_lines.prop_cycler)['color']
    ax.plot(DDplot.iloc[:,i], label=labels[i], color=color,linewidth=2)

ax.legend(fontsize=11)
ax.set_title('Drawdowns', fontsize=16)
vals = ax.get_yticks()
ax.set_yticklabels(['{:,.2%}'.format(x) for x in vals])

plt.grid()
plt.show()

#%% 12.2 Drawdown control: Treshold of 15% function

DD_treshold = pd.DataFrame(np.ones([DDplot.shape[0], 1]), columns=['Treshold'], index = DDplot.index)
DDplot      = pd.concat([DDplot.iloc[:,0],DD_treshold], axis=1)

for a in range(len(DDplot)):
    if   DDplot['Drawdown'] [a] < -0.15:
         DDplot['Treshold'] [a] = 0.5
        
#%% 12.3 Scale down the returns and cumulative return/index
tsmom_control = tsmom_ret_full.copy()
tsmom_control = pd.DataFrame(np.zeros([tsmom_ret_full.shape[0], 1]), columns=['Control'], index = tsmom_ret_full.index)

for a in range(len(tsmom_control)):
    tsmom_control['Control'][a] = DDplot['Treshold'][a] * tsmom_ret_full['Sum'][a]

#Add the first observation back again, so it's not NA
tsmom_control['Control'][0] = tsmom_ret_full['Sum'][0]
#Make cum and index
tsmom_control['Cum']       = (1 + tsmom_control['Control']).cumprod() -1 
tsmom_control['Cum Index'] = (1 + tsmom_control['Control']).cumprod()

#%% 12.4 Drawdown: TSMOM with & w/o DD-control
#Merge the cum returns into one dataframe
A24 = pd.concat([A23, tsmom_control['Cum']], axis=1)
A24.columns = ['Long', 'TSMOM', 'TSMOM DD-control']

#Create the figure
fig_strat_cum = A24.plot.line(x=None, y=None, \
                         title='Cumulative return with and without drawdown', figsize=(18,10)).grid(axis='y', color ='grey')               

## 12.4 Maximum on drawdown with control
V2  = tsmom_control.copy()
del V2['Control']
del V2['Cum']

DD2 = pd.DataFrame(np.zeros([V2.shape[0], V2.shape[1]]))

for i in range (0, V2.shape[1]):
    maxV2 = V2.iloc[1,i]
    for t in range(2, V2.shape[0]):
        DD2.iloc[t,i] = min(1.0, V2.iloc[t,i]/maxV2) - 1
        maxV2 = max(V2.iloc[t,i], maxV2)

max_dd2 = pd.DataFrame({'TSMOM': DD2.iloc[:,0].min()}, index=[0])

#%% 12.5 Visualization of TSMOM with and w/o DD-control

labels          = max_dd2.columns
V2              = V2.reset_index()
DDplot2         = pd.concat([V2.iloc[:,0],DD2], axis=1)
DDplot2.index   = pd.to_datetime(DDplot2.Date, format="%Y%m")
DDplot2         = DDplot2.drop(['Date'], axis=1)
DDplot2.columns = ['Drawdown']

fig, ax = plt.subplots(figsize=(10,6), dpi=100)

for i in range(0,DDplot2.shape[1]):
    color=next(ax._get_lines.prop_cycler)['color']
    ax.plot(DDplot2.iloc[:,i], label=labels[i], color=color,linewidth=2)

ax.legend(fontsize=11)
ax.set_title('Drawdowns', fontsize=16)
vals = ax.get_yticks()
ax.set_yticklabels(['{:,.2%}'.format(x) for x in vals])

plt.grid()
plt.show()

#%% 13.0 Sharpe on DD
DD_ret = tsmom_control.copy()
del DD_ret['Cum']
del DD_ret['Cum Index']

DD_sharpe = sharpe(DD_ret['Control'], ['TSMOM DD Control'])

#%% 13.1 DD regressed against benchmarks

#Make DD length as benchmarks
DD_ret.drop(DD_ret.tail( len(DD_ret)-len(all_bench) ).index, inplace=True )

#Regression with all benchmarks
Y4 = DD_ret
X = all_bench
X = sm.add_constant(X)
model       = sm.OLS(Y4,X)
results4 = model.fit()

#Regression with significant benchmarks
model    = sm.OLS(Y4, X3)
results5 = model.fit()





